#!/usr/bin/perl -w
# Author        : Pedro Velho
# last modified : 10/12/2008

$thisdir="/home/pvelho/07-speed/b-size/";    
if ( !(-d $thisdir) ){
    $thisdir=`pwd`;
    chomp($thisdir);
}
$workingdir=$thisdir;



# ****** 1- Parameter Message Size (S) ******
# in 100KB, 1MB, 2MB, 4MB, 8MB, 10MB, 20MB, 40MB, 80MB, 100MB 
# 
@message_size_list =("100000","1000000","2000000","4000000","8000000","10000000","20000000","40000000","80000000","100000000");

#******* 2- Number of Flows (N)    *****
@flows_list = (40);

# ****** 3- Experiments (E) ******
$exp=5;

# ****** 4- Model (M) ******
@model_list=("LV08FD", "GTNets");

# ****** 5- Duration (D) ******
#1, 2, 4, 8, 10, 20,  
#@duration_list=(40);
#, 80, 100

# ****** 6- Platform (P) ******
@platform_list=("waxman_50_1_converted.xml", "tiers_nodes_50_routers_35_converted.xml");

$task_begin=0;
$task_end=0;

# ~64K value maximum GTNetS value
$TCP_window=65535;

#########################################
# Parse and verify arguments range
#########################################
sub parseParameters() {
    #adjust to perl parameters (arrays are from 0 up size-1)
    $task_begin=$ARGV[1]-1;
    $task_end=$ARGV[2]-1;
    if($task_begin < 0){
	print "Starting from a not valuable array index aborting...\n";
	print "Check the correctness of first parameter.\n";
	exit 1;
    }
    print "File list size is : ",scalar($exp),"\n";
    if($task_end >= scalar($exp)){
	print "Error doing more tasks than we have to do aborting...\n";
	print "Check the correctness of second parameter.\n";
	exit 1;
    }
    if($task_begin > $task_end){
	print "Error starting point is greater then end point...\n";
	print "Second parameter must be greater.\n";
	exit 1;
    }
}




#########################################
# Print error and exit
#########################################
sub printError {
    my($errorMsg) = $_[0];
    print "Error : ".$errorMsg."\n";
    print "Parse usage:\n";
    print "\tsweep_analyze parse <task_begin> <task_end>\n\n";
    print "Sweep usage:\n";
    print "\tsweep_analyze sweep <task_begin> <task_end>\n\n";
    print "Examples\n";
    print "\tsweep_analyze 1 1 # executes task 1 (fix bandwidth[0] parameter)\n";
    print "\tsweep_analyze 1 3 # executes task from 1 up to 3\n\n";
    print "\nIn this program you have ".($exp)." tasks to parallelize.\n\n";
    exit 1;
}

if(scalar(@ARGV) < 1){ 
    printError("Needs at least one parameter sweep or parse");
}

if($ARGV[0] ne "sweep" && $ARGV[0] ne "parse"){
    printError("First argument should be sweep or parse");
}

if($ARGV[0] eq "sweep"){
    if(scalar(@ARGV) != 3){
	printError("Option sweep must be followed by two integers in task range.");
    }else {
	executeApplication();
    }
}

if($ARGV[0] eq "parse"){
    if(scalar(@ARGV) != 3){
	printError("Option parse must be followed by two integers in task range.");
    }else {
	parseParameters();
	parseResults(($task_begin+1), ($task_end+1));
    }
}



#########################################
# Execute application only once
#########################################
sub executeApplication {
    parseParameters();
    
    print "Changing working directory to ".$workingdir."\n";
    chdir($workingdir);

    $tmpoutputlog=$thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log";

    open(LOGOUTPUT, ">" , $tmpoutputlog) ||
	die "Sorry, couldn't open file ".$tmpoutputlog." for writting...\n";

    print "Loggin on file ".$tmpoutputlog."...\n";
    
    $it=0;
    for($it=$task_begin; $it <= $task_end; $it++) 
    {
	print "========> Experiment     (E) : ",$it,"\n";	    
	# Set parameters
	
	foreach $nflows (@flows_list) {	    
	    print "========> NFlows     (F) : ",$nflows,"\n";  
	    
	    foreach $size (@message_size_list){	    
		print "========> Size     (S) : ",$size,"\n";	    
#		    $duration = $duration_list[$it2];
#		    print "========> Duration (D) : ",$duration,"\n";	    
		
		create_deployment($size, $nflows);
   
		foreach $model (@model_list)
		{

		    print "========> Model     (M) : ",$model,"\n";		
		    
		    foreach $file (@platform_list)
		    {
			print "========> Platform  (P) : ",$file,"\n";		
			    
			#find deployment file depending on platform file name
			$deployment_file = $file;
			$deployment_file =~ s/(.*)\.xml$/$1/x;
			$deployment_file = $deployment_file."-d.xml";
			$temp_dep_file = $deployment_file."-size-".$size;
			    
			#generate new deployment for each experiment if not exists
			if( !( -r $thisdir."/deploy/".$temp_dep_file) ){
			    chdir($thisdir);
			    print $thisdir."/bin/generate_random_deployment_gtnets.pl ./platform/".$file." ".$nflows." ".$size."\n";
			    system($thisdir."/bin/generate_random_deployment_gtnets.pl ./platform/".$file." ".$nflows." ".$size."\n");
			    #save the experiment deployment file
			    system("cp ".$thisdir."/deploy/".$deployment_file." ".$thisdir."/deploy/".$temp_dep_file);
			    print("cp ".$thisdir."/deploy/".$deployment_file." ".$thisdir."/deploy/".$temp_dep_file."\n");
			    chdir($workingdir);
			}

			# Log some stuff into the file to know used parameters
			print LOGOUTPUT ">==================================================<\n";
			print LOGOUTPUT "========> Size          (S) : ",$size,"\n";	    
			print LOGOUTPUT "========> NFlows        (F) : ",$nflows,"\n";	     
			print LOGOUTPUT "========> Experiment    (E) : ",$it,"\n";
			print LOGOUTPUT "========> Model         (M) : ",$model,"\n";
			print LOGOUTPUT "========> Platform      (P) : ",$file,"\n";
			
			# Run with parameters without maxmin selective update
			#$if( $model =~ m/^CM02$/ ){
			    # 	$alpha=1;
			    # 	$beta=1;
			    # 	$s=0;		     
			    # 	$cmd=$workingdir."/gtnets ".$thisdir."/onelink-p.xml ".$thisdir."/deployment.xml --cfg=network/model:".$model." --cfg=fullduplex:0 --cfg=TCP_gamma:".$TCP_window." --cfg=network/latency_factor:".$alpha." --cfg=network/bandwidth_factor:".$beta." --cfg=network/weight_S:".$s." --cfg=maxmin_selective_update:0 2>&1";
			    # }elsif( $model =~ m/^LV08$/ ){
			    # 	$alpha=13.0076191103735;
			    # 	$beta=0.969832987216181;
			    # 	$s=0;		     
			    # 	$cmd=$workingdir."/gtnets ".$thisdir."/onelink-p.xml ".$thisdir."/deployment.xml  --cfg=network/model:".$model." --cfg=fullduplex:0 --cfg=TCP_gamma:".$TCP_window." --cfg=network/latency_factor:".$alpha." --cfg=network/bandwidth_factor:".$beta." --cfg=network/weight_S:".$s." --cfg=maxmin_selective_update:0 2>&1";
			    # }elsif( $model =~ m/^Reno$/ ){
			    # 	$alpha=13.0076191103735;
			    # 	$beta=0.969832987216181;
			    # 	$cmd=$workingdir."/gtnets ".$thisdir."/onelink-p.xml ".$thisdir."/deployment.xml  --cfg=network/model:Reno --cfg=fullduplex:0 --cfg=TCP_gamma:".$TCP_window." --cfg=network/latency_factor:".$alpha." --cfg=network/bandwidth_factor:".$beta." --cfg=maxmin_selective_update:0 2>&1";
#			    }elsif($model =~ m/LV08FD/ ){
			    if($model =~ m/LV08FD/ ){ 
				$alpha=13.0076191103735;
				$beta=0.969832987216181;
				$s=20537.14;		     
				$cmd=$thisdir."/gtnets ".$thisdir."/platform/".$file." ".$thisdir."/deploy/".$temp_dep_file." --cfg=network/model:LV08 --cfg=fullduplex:1 --cfg=TCP_gamma:".$TCP_window." --cfg=network/latency_factor:".$alpha." --cfg=network/bandwidth_factor:".$beta." --cfg=network/weight_S:".$s." --cfg=maxmin_selective_update:0 2>&1";
			    # }elsif($model =~ m/LV08FD_selective/ ){
			    # 	$alpha=13.0076191103735;
			    # 	$beta=0.969832987216181;
			    # 	$s=20537.14;		     
			    # 	$cmd=$workingdir."/gtnets ".$thisdir."/onelink-p.xml ".$thisdir."/deployment.xml --cfg=network/model:LV08 --cfg=fullduplex:1 --cfg=TCP_gamma:".$TCP_window." --cfg=network/latency_factor:".$alpha." --cfg=network/bandwidth_factor:".$beta." --cfg=network/weight_S:".$s." --cfg=maxmin_selective_update:1 2>&1";
			    }elsif( $model =~ m/^GTNets$/ ){
				$cmd=$thisdir."/gtnets ".$thisdir."/platform/".$file." ".$thisdir."/deploy/".$temp_dep_file." --cfg=network/model:".$model." --cfg=fullduplex:0 --cfg=TCP_gamma:".$TCP_window." 2>&1";
			    }	
			    
			    # Run with parameters
			    print("Running command:".$cmd."\n");
			    
			    # Open temporary output file
			    open(TMPOUTPUT, $cmd." | ") || die "Sorry, could't run application...\n";

			    # Copy execution trace into global trace
			    while(<TMPOUTPUT>)
			    {
				my $line = $_;
				chomp $line;
				print LOGOUTPUT $line."\n";
			    }
			    print LOGOUTPUT "=========================><=========================\n";
		    }			
		}
	    }
	}
    }
    

    print "==============================><=============================\n";   
    close(LOGOUTPUT);
}




#########################################
# Parse resulting values
# Genarate input file for R
#########################################
sub parseResults {
    my($begin_it, $end_it) = @_;
    if($begin_it == $end_it){
	open(RAWOUTPUT, ">".$thisdir."/dat/raw-".$begin_it.".data") ||
	    die "Sorry, couldn't open file ".$thisdir."/dat/raw-".$it.".data for writting...\n";
    }else{
	open(RAWOUTPUT, ">".$thisdir."/dat/raw-".$begin_it."-".$end_it.".data") ||
	    die "Sorry, couldn't open file ".$thisdir."/dat/raw-".$begin_it."-".$end_it.".data for writting...\n";
    }

    print RAWOUTPUT " File NFlow Size Experiment Model SimTime RunTime\n";



    $tracefile = $thisdir."/log/"."trace-file-".($begin_it)."-".($end_it).".log";

    unless (-e $tracefile) {
        print "File $tracefile doesn't exist, skiping...\n";
        next;
    } 

    open(LOGOUTPUT, "<", $tracefile) || die "Sorry, I coundn't open file ".$tracefile." for reading...";
    print $tracefile."\n";

    my($formater, $counter);
    my($File, $NFlow, $size, $Experiment, $Model, $simtime, $runtime);

    $counter = 1	;

	# Copy execution trace into global trace file
	while(<LOGOUTPUT>)
	{
	    my $line = $_;
	    chomp $line;
	    
	    if($line =~ m/^$/){
		print "Blank line found, skipping...\n";
	    }


	    # Simulated clock at finish time
	    if($line =~ /Simulated\sclock/){	
		$simtime = get_value($line);
	     	next;
	    }
	    
            # Simulation time
	    if($line =~ /Simulation\sTime/){	
		$runtime = get_value($line);
		print RAWOUTPUT $counter++." \"".$File."\" ".$NFlow." ".$size." ".$Experiment." \"".$Model."\" ".$simtime." ".$runtime."\n";
		next;	    
	    }

	    # Find a EXP label
	    if($line =~ /Experiment/){
		$Experiment = get_value($line);
		next;
	    }

	    # Find FILE
	    if($line =~ /Platform/){
		$File = $line;
		$File =~ s/^.*\(P\)\s*:\s*(.*)\s*$/$1/x;
		$File =~ s/^.*\/.*\/(.*)/$1/x;
		next;
	    }
	    
	    # Find model parameter
	    if($line =~ /Model/){
		$line =~ s/^.*\s+Model\s+\(M\)\s+:\s+(.*)\s*/$1/x;
		$Model = $line;
		next;
	    }

	    # Find Size
	    if($line =~ /Size/){
		$size = get_value($line);
		next;
	    }

	    # Find NFlow
	    if($line =~ /NFlow/){
		$NFlow = get_value($line);
		next;
	    }	    
	}
    close(LOGOUTPUT);
    close(RAWOUTPUT);
}
 

#########################################
# Parse numerical value within a line
#########################################
sub get_value {
    my($line)=shift;
    $line=" $line";

    $line =~ s/^
                 .*[^\d\.+-eE]+       # anything except something in a number
                 (
                   [+-]?              # first, match an optional sign
                   (                  # then match integers or f.p. mantissas:
                       \d+            # start out with a ...
                       (
                           \.\d*      # mantissa of the form a.b or a.
                       )?             # ? takes care of integers of the form a
                       |\.\d+         # mantissa of the form .b
                     )
                     ([eE][+-]?\d+)?  # finally, optionally match an exponent
                   )
		 \D*$/$1/x;
    return $line;
}


sub create_deployment() 
{
    my ($size, $nflow) = @_;

    open(DEPOUT, ">",  $thisdir."/deployment.xml") ||
	die "Sorry, couldn't open file deployment.xml for writting...\n";

    print DEPOUT    "<?xml version='1.0'?>\n";
    print DEPOUT	"<!DOCTYPE platform SYSTEM \"simgrid.dtd\">\n";
    print DEPOUT	"<platform version=\"2\">\n";
    print DEPOUT	"\n";

    for($i =0; $i<$nflow; $i++){	
	print DEPOUT	"  <process host=\"S1\" function=\"master\">\n";
	print DEPOUT	"      <argument value=\"".$size."\"/>\n";
	print DEPOUT	"      <argument value=\"C1\"/>\n";
	print DEPOUT	"      <argument value=\"".$i."\"/>\n";
	print DEPOUT	"  </process>\n";
	print DEPOUT	"  <process host=\"C1\" function=\"slave\">\n";
	print DEPOUT	"    <argument value=\"".$i."\"/>\n";
	print DEPOUT	"  </process>\n";
    } 

    print DEPOUT "\n";
    print DEPOUT "</platform>\n";

    close(DEPOUT);
}

#########################################
# Erase undesired characters at the 
# end of string
#########################################
sub trim 
{
    my @out = @_;
    for (@out) 
    {
        s/^\s+//;
        s/\s+$//;
    }
    return wantarray ? @out : $out[0];
}
