#!/usr/bin/perl -w
use strict;

sub read_file {
    my($filename)=shift;
    my($line);
    my(@host_list);
    open(INPUT,"$filename") || die "Impossible to open platform file \"$filename\"\n";
    while(defined($line=<INPUT>)) {
	chomp $line;
	if($line=~/host id/) {
	    $line=~ s/.*host id="//;
	    $line=~ s/\".*$//;
	    push @host_list,$line;
	}
    }
    close(INPUT);
    return \@host_list;
}

sub generate_random_deployment{
    my($host_list,$nflows,$filename,$size)=@_;
    my(%pairs);
    my($nhost) = scalar(@$host_list);

    $nflows< $nhost*$nhost-$nhost or die "Too much flows! I can't do it\n";
   
    while(scalar(keys(%pairs))<$nflows) {
	my($src)=int(rand(scalar(@$host_list)));
	my($dst)=int(rand(scalar(@$host_list)));

	if($src!=$dst && !defined($pairs{"$$host_list[$src] $$host_list[$dst]"})) {
	    $pairs{"$$host_list[$src] $$host_list[$dst]"}=1;
	}
    }

    open(CHUBS,"> $filename") || die "Impossible to open output file \"$filename\"\n";

    my($p,$count);
    print CHUBS "<?xml version='1.0'?>\n";
    print CHUBS "<!DOCTYPE platform SYSTEM \"simgrid.dtd\">\n";
    print CHUBS "<platform version=\"2\">\n\n";   
    $count = 1;
    foreach $p (keys %pairs) {
	my($src,$dst)=split(/ /,$p);
	print CHUBS "  <process host='$src' function='master'>\n";
	print CHUBS "    <argument value='$size'/>\n";
	print CHUBS "    <argument value='$dst'/>\n";
	print CHUBS "    <argument value='$count'/>\n";
	print CHUBS "  </process>\n";
	print CHUBS "  <process host='$dst' function='slave'>\n";
	print CHUBS "    <argument value='$count'/>\n";
	print CHUBS "  </process>\n\n";
	$count++;
    }
    print CHUBS "</platform>\n";

    close(CHUBS);
}

sub main {
    my($nodes,$edges,$interferences,$host_list,$count_interferences);
   
    $#ARGV>=2 or die "Usage:\n\t./generate_random_deployment.pl <platform_file> <number_of_flows> <message_size>\n\n";
    my($filename)=$ARGV[0];
    my($nflows)=$ARGV[1];
    my($size)=$ARGV[2];

    my($thisdir, @file_with_path);
    $thisdir=`pwd`;
    chomp $thisdir;
    

    # Cut off file path
    @file_with_path = split("/",$filename);
    $filename = $file_with_path[scalar(@file_with_path)-1];

    $filename =~ s/\.xml$//g;
    $filename =~ s/-p$//g;
    $filename = $thisdir."/deploy/".$filename."-d.xml";

    $host_list = read_file $ARGV[0];
    generate_random_deployment($host_list,$nflows,$filename,$size);
}

main;
