
plotDoubleGraphWithCI = function(X, Y1, minY1, maxY1, Y2, minY2, maxY2, minX=0, maxX=0, ticsX=0, minY = 0, maxY = 0, ticsY = 0,  xLabel, yLabel, labelX, labelY, exportSvg=FALSE){

  tmpFrame = data.frame(as.numeric(X), as.numeric(Y1), as.numeric(minY1), as.numeric(maxY1), as.numeric(Y2), as.numeric(minY2), as.numeric(maxY2))

  dataFileName = "tmp/doubleGraphWithCI.data"
  write.table(file=dataFileName, row.names=FALSE, col.names=FALSE, tmpFrame)
  outFile = file("tmp/doubleGraphWithCI.script", "w")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set terminal svg enhanced size 640 480 fname \"Arial\" fsize 15 \n")
    cat(file = outFile, sep = "", "set output './tmp/doubleGraphWithCI.svg'\n")
  }


  cat(file=outFile, "set logscale xy\n",sep="")  

  if((minX != 0) && (maxX != 0)){
    cat(file=outFile, "set xrange [",minX,":",maxX,"]\n",sep="")
  }

  if(ticsX != 0){
    cat(file=outFile, "set xtics ",ticsX,"\n",sep="")
  }

  if(minY != 0 && maxY !=0){
    cat(file=outFile, "set yrange [",minY,":",maxY,"]\n",sep="")
  }
  

  if(ticsY != 0){
    cat(file=outFile, "set ytics ",ticsY,"\n",sep="")
  }

  if(xLabel != 0){
    cat(file=outFile, "set xlabel \"",xLabel,"\" \n",sep="")
  }

  if(yLabel != 0){
    cat(file=outFile, "set ylabel \"",yLabel,"\" \n",sep="")
  }
  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "set key left top\n",sep="")

  cat(file=outFile, "plot '",dataFileName,"' u 1:2:3:4 w yerrorbars   lt rgb \"#000000\" t '' , '",
                             dataFileName,"' u 1:2 w linespoints lw 2 lt rgb \"#cc3333\" t '",labelX,"', '",
                             dataFileName,"' u 1:5:6:7 w yerrorbars   lt rgb \"#000000\" t '' , '",
                             dataFileName,"' u 1:5 w linespoints lw 2 lt rgb \"#6666cc\" t '",labelY,"' \n",sep="")
  close(outFile)
  system("gnuplot -persist tmp/doubleGraphWithCI.script")
}



#############################################
#############################################

plotTripleGraphWithCI = function(X, Y1, minY1, maxY1, Y2, minY2, maxY2, Y3, minY3, maxY3, minX=0, maxX=0, ticsX=0, minY = 0, maxY = 0, ticsY = 0,  xLabel, yLabel, labelX, labelY, labelZ, exportSvg=FALSE){

  tmpFrame = data.frame(as.numeric(X), as.numeric(Y1), as.numeric(minY1), as.numeric(maxY1), as.numeric(Y2), as.numeric(minY2), as.numeric(maxY2), as.numeric(Y3), as.numeric(minY3), as.numeric(maxY3))

  dataFileName = "tmp/doubleGraphWithCI.data"
  write.table(file=dataFileName, row.names=FALSE, col.names=FALSE, tmpFrame)
  outFile = file("tmp/doubleGraphWithCI.script", "w")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set terminal svg enhanced size 640 480 fname \"Arial\" fsize 15 \n")
    cat(file = outFile, sep = "", "set output './tmp/doubleGraphWithCI.svg'\n")
  }

  cat(file=outFile, "set logscale xy\n",sep="")  

  if((minX != 0) && (maxX != 0)){
    cat(file=outFile, "set xrange [",minX,":",maxX,"]\n",sep="")
  }

  if(ticsX != 0){
    cat(file=outFile, "set xtics ",ticsX,"\n",sep="")
  }

  if(minY != 0 && maxY !=0){
    cat(file=outFile, "set yrange [",minY,":",maxY,"]\n",sep="")
  }

  if(ticsY != 0){
    cat(file=outFile, "set ytics ",ticsY,"\n",sep="")
  }

  if(xLabel != 0){
    cat(file=outFile, "set xlabel \"",xLabel,"\" \n",sep="")
  }

  if(yLabel != 0){
    cat(file=outFile, "set ylabel \"",yLabel,"\" \n",sep="")
  }
  
  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "set key left top\n",sep="")

  cat(file=outFile, "plot '",dataFileName,"' u 1:2:3:4 w yerrorbars   lt rgb \"#000000\" t '' , '",
                             dataFileName,"' u 1:2 w linespoints lw 2 lt rgb \"#cc3333\" t '",labelX,"', '",

                             dataFileName,"' u 1:5:6:7 w yerrorbars   lt rgb \"#000000\" t '' , '",
                             dataFileName,"' u 1:5 w linespoints lw 2 lt rgb \"#6666cc\" t '",labelY,"', '", 

                             dataFileName,"' u 1:9:10:11 w yerrorbars   lt rgb \"#000000\" t '' , '",
                             dataFileName,"' u 1:9 w linespoints lw 2 lt rgb \"#99cc99\" t '",labelZ,"' \n",sep="")
  close(outFile)
  system("gnuplot -persist tmp/doubleGraphWithCI.script")
}





##################################
##################################
##################################
# Graphs and numbers to support
# main text
##################################
##################################
##################################
 
#############################################################################
# Plot a 3D graph with T = f(S,L) for a fixed bandwidth of 56 kbit/s (7000 Bps)
# ./figures/ch6-02.pdf
#############################################################################
resultTable  = read.table("./dat/raw-1-5.data")
tb = resultTable

temp1  = tb[((tb$Model == 'CM02')&(tb$File == 'waxman_50_1_converted.xml')&(tb$Experiment == 0)),]
temp1  = tb[((tb$Model == 'LV08')&(tb$File == 'waxman_50_1_converted.xml')&(tb$Experiment == 0)),]
temp1  = tb[((tb$Model == 'LV08FD')&(tb$File == 'waxman_50_1_converted.xml')&(tb$Experiment == 0)),]
temp1  = tb[((tb$Model == 'Reno')&(tb$File == 'waxman_50_1_converted.xml')&(tb$Experiment == 0)),]

len = length(unique(tb$Model))*length(unique(tb$NFlow))

File          = seq(0, length=len, by=0)
NFlows        = seq(0, length=len, by=0)
Model         = seq(0, length=len, by=0)
Sum           = seq(0, length=len, by=0)
Mean          = seq(0, length=len, by=0)
Median        = seq(0, length=len, by=0)
Min           = seq(0, length=len, by=0)
Max           = seq(0, length=len, by=0)
Error         = seq(0, length=len, by=0)

rm(gloFrame)
gloFrame = data.frame(File, NFlows, Model, Sum, Mean, Median, Min, Max, Error)
glFrameIndex = 0

for(file  in unique(tb$File) ){
  for(model  in unique (tb$Model) ){
    for(nf  in unique(tb$NFlow) ){
      
      tmp = tb[((tb$Model == model)&(tb$NFlow == nf)&(tb$File == file)),]
      
      glFrameIndex = glFrameIndex + 1
      
      th = as.numeric(tmp$RunTime)

      gloFrame[glFrameIndex,1]  = file
      gloFrame[glFrameIndex,2]  = nf
      gloFrame[glFrameIndex,3]  = model
      gloFrame[glFrameIndex,4]  = sum(th)

      
      if(length(th) == 1){
        gloFrame[glFrameIndex,5]  = th
        gloFrame[glFrameIndex,6]  = th
        gloFrame[glFrameIndex,7]  = th
        gloFrame[glFrameIndex,8]  = th
        gloFrame[glFrameIndex,9]  = 0
      }else{
        gloFrame[glFrameIndex,5]  = mean(th)
        gloFrame[glFrameIndex,6]  = median(th)
        gloFrame[glFrameIndex,7]  = min(th)
        gloFrame[glFrameIndex,8]  = max(th)   
        
        m = mean(th)
        s = sd(th)
        n = length(th)
        error = qt(0.975,df=n-1)*s/sqrt(n)
        
        gloFrame[glFrameIndex,9]  = error
      }
    }
  }
}

             
t1 = gloFrame[((gloFrame$Model == 'CM02')&(gloFrame$File == 'waxman_50_1_converted.xml')),]
t2 = gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$File == 'waxman_50_1_converted.xml')),]

plotDoubleGraphWithCI(t1$NFlow, t1$Mean, t1$Mean-t1$Error, t1$Mean+t1$Error, t2$Mean, t2$Mean-t2$Error, t2$Mean+t2$Error, minX=9, maxX=352, ticsX="(10,20,40,80,160,320)", minY=0, maxY=0, ticsY=0, xLabel="number of simulated flows", yLabel="execution time (s)", labelX="  SimGrid  ", labelY="  FullDuplex  ", exportSvg=FALSE)

t1 = gloFrame[((gloFrame$Model == 'CM02')&(gloFrame$File == 'tiers_nodes_50_routers_35_converted.xml')),]
t2 = gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$File == 'tiers_nodes_50_routers_35_converted.xml')),]

plotDoubleGraphWithCI(t1$NFlow, t1$Mean, t1$Mean-t1$Error, t1$Mean+t1$Error, t2$Mean, t2$Mean-t2$Error, t2$Mean+t2$Error, minX=9, maxX=352, ticsX="(10,20,40,80,160,320)", minY=0, maxY=0, ticsY=0, xLabel="number of simulated flows", yLabel="execution time (s)", labelX="  SimGrid  ", labelY="  FullDuplex  ", exportSvg=FALSE)


t1 = gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$File == 'waxman_50_1_converted.xml')),]
t2 = gloFrame[((gloFrame$Model == 'Reno')&(gloFrame$File == 'waxman_50_1_converted.xml')),]

plotDoubleGraphWithCI(t1$NFlow, t1$Mean, t1$Mean-t1$Error, t1$Mean+t1$Error, t2$Mean, t2$Mean-t2$Error, t2$Mean+t2$Error, minX=9, maxX=352, ticsX="(10,20,40,80,160,320)", minY=0, maxY=0, ticsY=0, xLabel="number of simulated flows", yLabel="execution time (s)", labelX="  FullDuplex  ", labelY="  Reno  ", exportSvg=FALSE)


t1 = gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$File == 'tiers_nodes_50_routers_35_converted.xml')),]
t2 = gloFrame[((gloFrame$Model == 'Reno')&(gloFrame$File == 'tiers_nodes_50_routers_35_converted.xml')),]

plotDoubleGraphWithCI(t1$NFlow, t1$Mean, t1$Mean-t1$Error, t1$Mean+t1$Error, t2$Mean, t2$Mean-t2$Error, t2$Mean+t2$Error, minX=9, maxX=352, ticsX="(10,20,40,80,160,320)", minY=0, maxY=0, ticsY=0, xLabel="number of simulated flows", yLabel="execution time (s)", labelX="  FullDuplex  ", labelY="  Reno  ", exportSvg=FALSE)



t1 = gloFrame[((gloFrame$Model == 'Reno')&(gloFrame$File == 'waxman_50_1_converted.xml')),]
t2 = gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$File == 'waxman_50_1_converted.xml')),]
t3 = gloFrame[((gloFrame$Model == 'CM02')&(gloFrame$File == 'waxman_50_1_converted.xml')),]

plotTripleGraphWithCI(t1$NFlow, t1$Mean, t1$Mean-t1$Error, t1$Mean+t1$Error, t2$Mean, t2$Mean-t2$Error, t2$Mean+t2$Error, t3$Mean, t3$Mean-t3$Error, t3$Mean+t3$Error, minX=9, maxX=352, ticsX="(10,20,40,80,160,320)", minY=0, maxY=0, ticsY=0, xLabel="number of simulated flows", yLabel="execution time (s)", labelX="  Reno  ", labelY="  Fullduplex  ", labelZ="  SimGrid  ", exportSvg=FALSE)


t1 = gloFrame[((gloFrame$Model == 'LV08FD')&(gloFrame$File == 'tiers_nodes_50_routers_35_converted.xml')),]
t2 = gloFrame[((gloFrame$Model == 'CM02')&(gloFrame$File == 'tiers_nodes_50_routers_35_converted.xml')),]
t3 = gloFrame[((gloFrame$Model == 'Reno')&(gloFrame$File == 'tiers_nodes_50_routers_35_converted.xml')),]

plotTripleGraphWithCI(t1$NFlow, t1$Mean, t1$Mean-t1$Error, t1$Mean+t1$Error, t2$Mean, t2$Mean-t2$Error, t2$Mean+t2$Error, t3$Mean, t3$Mean-t3$Error, t3$Mean+t3$Error, minX=9, maxX=352, ticsX="(10,20,40,80,160,320)", minY=0, maxY=0, ticsY=0, xLabel="number of simulated flows", yLabel="execution time (s)", labelX="  FullDuplex  ", labelY="  SimGrid  ", labelZ="  Reno  ", exportSvg=TRUE)
