/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <semaphore.h>
#include "msg/msg.h"
#include "xbt/log.h"
#include "xbt/asserts.h"

XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
                             "Messages specific for this msg example");

int master(int argc, char *argv[]);
int slave(int argc, char *argv[]);
static int timer(int argc, char *argv[]);
MSG_error_t test_all(const char *platform_file,
                     const char *application_file);


typedef enum {
  PORT_22 = 0,
  MAX_CHANNEL
} channel_t;


//keep a pointer to all surf running tasks.
#define NTASKS 200000
int bool_printed = 0;
double start_time, end_time, elapsed_time;
double gl_data_size[NTASKS];
m_task_t gl_task_array[NTASKS];
const char *slavenames[NTASKS];
const char *masternames[NTASKS];
int gl_task_array_id = 0;
double stop_simulation = 10.0;
struct timeval start, finish;
sem_t create_timer;
int timer_created = 0;
int message_size=121;


#define FINALIZE ((void*)221297)        /* a magic number to tell people to stop working */


/*
 * Sender 
 */
int master(int argc, char *argv[])
{
  char *slavename = NULL;
  m_task_t todo;
  m_host_t slave;
  char id_alias[10];
  //unique id to control statistics
  int id = -1;
  int temp_size = 0;

  m_process_t timer_proc = NULL;

  sem_wait(&create_timer);
  if(!timer_created){
    timer_created = 1;
    sem_post(&create_timer);
    INFO0("Creating timer...");
    timer_proc = MSG_process_create("timer", timer, NULL, MSG_host_self());
    INFO1("Timer created by process %p ...", MSG_host_self());
  }else{
    sem_post(&create_timer);
  }

  if (argc != 4) {
    INFO1("Strange number of arguments expected 4 got %d", argc);
  }

  /* slave name */
  slavename = argv[2];
  id = atoi(argv[3]);
  sprintf(id_alias, "flow_%d", id);
  slavenames[id] = slavename;
  TRACE_category(id_alias);

  masternames[id] = MSG_host_get_name(MSG_host_self());

  slave = MSG_get_host_by_name(slavename);
  
  /* time measurement */
  sprintf(id_alias, "%d", id);
  start_time = MSG_get_clock();

  while(1){
    char sprintf_buffer[64] = "Task_0";
    temp_size = message_size + (message_size*(rand()%5));
    //INFO2("Sending message of size %d, rand %d", temp_size, rand()%6);
    todo = MSG_task_create(sprintf_buffer, 0, temp_size , NULL);
    MSG_task_send(todo, id_alias);
    if(MSG_get_clock() > stop_simulation){
      return 0;
    }
  }

  end_time = MSG_get_clock();
  return 0;
}


/*
 * Receiver function  
 */
int slave(int argc, char *argv[])
{

  m_task_t task = NULL;
  int a;
  int id = 0;
  char id_alias[10];

  if (argc != 2) {
    INFO1("Strange number of arguments expected 2 got %d", argc);
  }

  id = atoi(argv[1]);
  sprintf(id_alias, "%d", id);

  while(1){
	  a = MSG_task_receive(&(task), id_alias);
	  //INFO0("Received message");
	  MSG_task_destroy(task);
	  task = NULL;
	  if(MSG_get_clock() > stop_simulation){
	    return 0;
	  }
  }

  MSG_task_destroy(task);
  return 0;
}


/*
 * Create a time that trigger's the end of the simulation after duration 
 */
static int timer(int argc, char *argv[]){
	double time;
	INFO1("Timer process going for a nap %f", stop_simulation);
	MSG_process_sleep(stop_simulation);
	gettimeofday(&finish, NULL);
	time = start.tv_sec + start.tv_usec*1.0e-6;
	time = (finish.tv_sec + finish.tv_usec*1.0e-6) - time;
	INFO1("Simulated clock: %f", MSG_get_clock());
	INFO1("Simulation Time: %f seconds", time);
	exit(0);
}


/*
 * Test function 
 */
MSG_error_t test_all(const char *platform_file,
                     const char *application_file)
{
  MSG_error_t res = MSG_OK;
  /*  Simulation setting */
  MSG_set_channel_number(MAX_CHANNEL);
  MSG_create_environment(platform_file);
  /*   Application deployment */
  MSG_function_register("master", master);
  MSG_function_register("slave", slave);
  MSG_function_register("timer", timer);
  MSG_launch_application(application_file);
  gettimeofday(&start, NULL);
  res = MSG_main();
  return res;
}


/*
 * Main function 
 */
int main(int argc, char *argv[])
{
  MSG_error_t res = MSG_OK;
  
  MSG_global_init(&argc, argv);
  if (argc <= 5) {
    INFO1("Usage: %s <platform_file> <deployment_file> <stop_timer> <size> <seed>", argv[0]);
    exit(1);
  }
  message_size = atoi(argv[4]);
  INFO1("Setting message size to %d", message_size);
  int seed = atoi(argv[5]);
  INFO1("Setting seed to %d", seed);
  srand(seed);
  stop_simulation = atof(argv[3]);
  sem_init(&create_timer, 0, 1);
  res = test_all(argv[1], argv[2]);
  MSG_clean();
  if (res == MSG_OK){
    return 0;
  }else{
    return 1;
  }
}
