gnuplot3d = function (X,Y,Z,W,minX=1,maxX=5,minY=0,maxY=5,minZ=0,maxZ=5,labelX="X",labelY="Y",labelZ="Z", ticsX="", ticsY="", ticsZ="", f="x + y", title="Default Title", exportSvg = FALSE) {
  #export data to gnuplot file
  outFile = file("./tmp/gnuplot3d.data", "w")
  cat("#",labelX,labelY,labelZ,"\n",sep=" ")
  write.table(data.frame(X,Y,Z), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)
   
  #export gnuplot script
  outFile = file("./tmp/gnuplot3d.script", "w")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")
  cat(file = outFile, sep = "", "set zrange [",minZ, ":",maxZ,"]\n")
  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")
  cat(file = outFile, sep="", "set zlabel \"",labelZ,"\"\n")  

  if(ticsX != ""){
    cat(file = outFile, sep="", "set xtics ",ticsX,"\n")
  }

  if(ticsY != ""){
    cat(file = outFile, sep="", "set ytics ",ticsY,"\n")
  }

  if(ticsZ != ""){
    cat(file = outFile, sep="", "set ztics ",ticsZ,"\n")
  }

  cat(file=outFile, "set logscale zx\n",sep="")  

  
  cat(file = outFile, sep="", "unset key\n")
  cat(file = outFile, sep="", "set title \"", title, "\"\n")
  cat(file = outFile, sep="", "min(x,y)=(x<y)?x:y\n")
  cat(file = outFile, sep="", "f(x,y) = ",f, "\n")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set term svg \n")
    cat(file = outFile, sep = "", "set output './tmp/gnuplot3d.svg'\n")
  }

  cat(file = outFile, "splot './tmp/gnuplot3d.data' u 1:2:3 ls 7 lt rgb \"#000000\" t 'GTNetS' \n")

  close(outFile)

  #call gnuplot and present 3D data
  system("gnuplot -persist ./tmp/gnuplot3d.script")
}

plotDoubleGraphWithCI = function(X, Y, minY, maxY, Z, minBottomGraph = 0, maxBottomGraph = 0, ticsBottomGraph=0, minX=0, maxX=0, ticsX=0, minTopGraph = 0, maxTopGraph = 0, ticsTopGraph=0, orderBy=1, titleTop="Top graph", titleBottom="Bottom graph", xTitle=0, yTitle=0, yLabel=0, max, exportSvg=FALSE){

  tmpFrame = data.frame(as.numeric(X), as.numeric(Y), as.numeric(minY), as.numeric(maxY), as.numeric(Z))
  
  dataFileName = "tmp/doubleGraphWithCI.data"
  write.table(file=dataFileName, row.names=FALSE, col.names=FALSE, tmpFrame)
  outFile = file("tmp/doubleGraphWithCI.script", "w")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set terminal svg enhanced size 640 480 fname \"Arial\" fsize 15 \n")
    cat(file = outFile, sep = "", "set output './tmp/doubleGraphWithCI.svg'\n")
  }

  
  cat(file=outFile, "unset key  \n",sep="")

  if((minX != 0) && (maxX != 0)){
    cat(file=outFile, "set xrange [",minX,":",maxX,"]\n",sep="")
  }

  if(ticsX != 0){
    cat(file=outFile, "set xtics ",ticsX,"\n",sep="")
  }

  cat(file=outFile, "set lmargin 10\n",sep="")
  cat(file=outFile, "set rmargin 2\n",sep="")
  cat(file=outFile, "set multiplot\n",sep="")
  cat(file=outFile, "set logscale x\n",sep="")  

  if(xTitle != 0){
    cat(file=outFile, "set xlabel \"",xTitle,"\" \n",sep="")
  }

  if(yLabel != 0){
    cat(file=outFile, "set ylabel \"",yLabel,"\" \n",sep="")
  }
  
  cat(file=outFile, "set size 1,0.4\n",sep="")
  cat(file=outFile, "set origin 0.0,0.0\n",sep="")

  if(minBottomGraph != 0 && maxBottomGraph !=0){
    cat(file=outFile, "set yrange [",minBottomGraph,":",maxBottomGraph,"]\n",sep="")
  }
  
  if(ticsBottomGraph != 0){
    cat(file=outFile, "set ytics ",ticsBottomGraph,"\n",sep="")
  }

  cat(file=outFile, "set tmargin 1\n",sep="")
  cat(file=outFile, "plot '",dataFileName,"' u 1:5 w points lt rgb \"#000000\" , ",max,"\n",sep="")
  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "set size 1,0.6\n",sep="")

  if(yTitle != 0){
      cat(file=outFile, "set ylabel \"",yTitle,"\" \n",sep="")
  }

  if( maxTopGraph !=0){
    cat(file=outFile, "set yrange [",minTopGraph,":",maxTopGraph,"]\n",sep="")
  }  

  if(ticsTopGraph != 0){
    cat(file=outFile, "set ytics ",ticsTopGraph,"\n",sep="")
  }
  
  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "unset xtics\n",sep="")
  cat(file=outFile, "set logscale xy\n",sep="")  
  
  cat(file=outFile, "set origin 0.0,0.4\n",sep="")
  cat(file=outFile, "set format x \"\"\n",sep="")
  cat(file=outFile, "set xlabel \"\"\n",sep="")
  cat(file=outFile, "plot '",dataFileName,"' u 1:2:3:4 w yerrorbars lt rgb \"#000000\" t \"",titleBottom,"\" , ",max,"/x t 'f(x)  = ",max,"/x'\n",sep="")
  cat(file=outFile, "set nomultiplot\n",sep="")
  close(outFile)
  system("gnuplot -persist tmp/doubleGraphWithCI.script")
}



##################################
##################################
##################################
# Graphs and numbers to support
# main text
##################################
##################################
##################################
 
#############################################################################
# Plot a 3D graph with T = f(S,L) for a fixed bandwidth of 56 kbit/s (7000 Bps)
# ./figures/ch6-02.pdf
#############################################################################
cpuResult  = read.table("./cpu_sharing/raw.single.data")
tb = cpuResult
len = length(unique(tb$nThreads));

nThreads      = seq(0, length=len, by=0)
Sum           = seq(0, length=len, by=0)
Mean          = seq(0, length=len, by=0)
Median        = seq(0, length=len, by=0)
Min           = seq(0, length=len, by=0)
Max           = seq(0, length=len, by=0)
Error         = seq(0, length=len, by=0)


rm(gloFrame)
gloFrame = data.frame(nThreads, Sum, Mean, Median, Min, Max, Error)
glFrameIndex = 0
for(NT  in unique (tb$nThreads) ){
  th = tb[((tb$nThreads == NT)),]$flops / tb[((tb$nThreads == NT)),]$time
  glFrameIndex = glFrameIndex + 1
  
  gloFrame[glFrameIndex,1]  = NT
  gloFrame[glFrameIndex,2]  = sum(th)


  if(length(th) == 1){
    gloFrame[glFrameIndex,3]  = th
    gloFrame[glFrameIndex,4]  = th
    gloFrame[glFrameIndex,5]  = th
    gloFrame[glFrameIndex,6]  = th
    gloFrame[glFrameIndex,7]  = 0
  }else{
    gloFrame[glFrameIndex,3]  = mean(th)
    gloFrame[glFrameIndex,4]  = median(th)
    gloFrame[glFrameIndex,5]  = min(th)
    gloFrame[glFrameIndex,6]  = max(th)   

    m = mean(th)
    s = sd(th)
    n = length(th)
    error = qt(0.975,df=n-1)*s/sqrt(n)

    gloFrame[glFrameIndex,7]  = error
  }
}

             
t = gloFrame
plotDoubleGraphWithCI(t$nThreads, t$Mean, t$Mean-t$Error, t$Mean+t$Error, t$Sum, minX=0.9, maxX=400, minBottomGraph = 1, maxBottomGraph = 28,  minTopGraph=0.01, maxTopGraph=28, ticsTopGraph="(\"100\" 25, \"50\" 12.5 , \"10\" 2.5 , \"1\" 0.25 )", ticsBottomGraph="(\"100\" 25, \"50\" 12.5)", ticsX="(1,2,3,4,5, 10, 20 , 30 ,40 ,50 ,100,200,300)", max=25, exportSvg=FALSE)




##################################
##################################
##################################
# Graphs and numbers to support
# main text
##################################
##################################
##################################
 
#############################################################################
# Plot a 3D graph with T = f(S,L) for a fixed bandwidth of 56 kbit/s (7000 Bps)
# ./figures/ch6-02.pdf
#############################################################################
cpuResult  = read.table("./cpu_sharing/raw.memory.data")
tb = cpuResult
len = length(unique(tb$nThreads));

nThreads      = seq(0, length=len, by=0)
Sum           = seq(0, length=len, by=0)
Mean          = seq(0, length=len, by=0)
Median        = seq(0, length=len, by=0)
Min           = seq(0, length=len, by=0)
Max           = seq(0, length=len, by=0)
Error         = seq(0, length=len, by=0)


rm(gloFrame)
gloFrame = data.frame(nThreads, Sum, Mean, Median, Min, Max, Error)
glFrameIndex = 0
for(NT  in unique (tb$nThreads) ){
  th = tb[((tb$nThreads == NT)),]$flops / tb[((tb$nThreads == NT)),]$time
  glFrameIndex = glFrameIndex + 1
  
  gloFrame[glFrameIndex,1]  = NT
  gloFrame[glFrameIndex,2]  = sum(th)


  if(length(th) == 1){
    gloFrame[glFrameIndex,3]  = th
    gloFrame[glFrameIndex,4]  = th
    gloFrame[glFrameIndex,5]  = th
    gloFrame[glFrameIndex,6]  = th
    gloFrame[glFrameIndex,7]  = 0
  }else{
    gloFrame[glFrameIndex,3]  = mean(th)
    gloFrame[glFrameIndex,4]  = median(th)
    gloFrame[glFrameIndex,5]  = min(th)
    gloFrame[glFrameIndex,6]  = max(th)   

    m = mean(th)
    s = sd(th)
    n = length(th)
    error = qt(0.975,df=n-1)*s/sqrt(n)

    gloFrame[glFrameIndex,7]  = error
  }
}

             
t = gloFrame
plotDoubleGraphWithCI(t$nThreads, t$Mean, t$Mean-t$Error, t$Mean+t$Error, t$Sum, minX=0.9, maxX=400, minBottomGraph = 1, maxBottomGraph = 8000,  minTopGraph=1, maxTopGraph=8000, ticsTopGraph="(\"100\" 6900, \"50\" 3950 , \"10\" 690 , \"1\" 69 )", ticsBottomGraph="(\"100\" 6900, \"50\" 3950)", ticsX="(1,2,3,4,5, 10, 20 , 30 ,40 ,50 ,100,200,300)", max=6900, exportSvg=TRUE )


